'use strict';

var binary_hamming_distance = require('../_virtual/binary_hamming_distance.cjs');

/* eslint-disable no-bitwise */
const bitCount = (n) => {
    let result = 0;
    let tmp = n;
    while (tmp) {
        result += 1;
        tmp &= tmp - 1;
    }
    return result;
};
const bigIntBitCount = (n) => {
    let result = 0;
    let tmp = n;
    while (tmp) {
        result += 1;
        tmp &= tmp - 1n;
    }
    return result;
};
/**
 * Hamming distance between two strings or numbers
 * @param a - first string or number e.g. '123456'
 * @param b - second string or number e.g. '654321'
 */
const decimalHammingDistance = (a, b) => {
    const res = Number(a) ^ Number(b);
    return bitCount(res);
};
const bigIntHammingDistance = (a, b) => {
    const res = a ^ b;
    return bigIntBitCount(res);
};
/**
 * Hamming distance between two binary strings
 * @param a - first string e.g. '101010'
 * @param b - second string e.g. '010101'
 */
const binaryHammingDistance = (a, b) => {
    if (a.length !== b.length) {
        throw new Error('Hamming distance between two binary strings requires equal length strings');
    }
    let distance = 0;
    for (let i = 0; i < a.length; i++) {
        if (a[i] !== b[i]) {
            distance += 1;
        }
    }
    return distance;
};
/**
 * Hamming distance between two binary strings
 * @param a - first string e.g. '101010'
 * @param b - second string e.g. '010101'
 */
const binaryStringHammingDistance = (a, b) => {
    const _a = BigInt(`0b${a}`);
    const _b = BigInt(`0b${b}`);
    const res = _a ^ _b;
    return bigIntBitCount(res);
};
const wasmBinaryStringHammingDistance = (a, b) => {
    return binary_hamming_distance.binary_hamming_distanceExports.binary_hamming_distance(a, b);
};

exports.bigIntBitCount = bigIntBitCount;
exports.bigIntHammingDistance = bigIntHammingDistance;
exports.binaryHammingDistance = binaryHammingDistance;
exports.binaryStringHammingDistance = binaryStringHammingDistance;
exports.bitCount = bitCount;
exports.decimalHammingDistance = decimalHammingDistance;
exports.wasmBinaryStringHammingDistance = wasmBinaryStringHammingDistance;
//# sourceMappingURL=hammingDistance.cjs.map
